/******************************************************************************
* To the extent possible under law, Sebastian Roith
* <https://www.roith-unterrichtet.de> has waived all copyright and related or
* neighboring rights to this script. This work is published from Germany.
******************************************************************************/

/**
* Klasse, die __eine__ Frage auf dem Bildschirm darstellt und auswertet
*/
function FragenUI(
    data,       // Daten und Einstellungen dieser Frage
    nodeParent, // HTMLElement, in dem die Frage angezeigt werden soll
    intNumber   // Nummer der Frage
) {
    this.data = data;
    this._choices = [];
    this._establish(nodeParent, intNumber);
}

// Daten und Einstellungen dieser Frage
FragenUI.prototype.data = {};

/**
* Frage als HTML anlegen
* @return undefined
*/
FragenUI.prototype._establish = function(
    nodeParent, // HTMLElement, in dem die Frage angezeigt werden soll
    intNumber   // Nummer der Frage
) {
    var node = AudioFeedbackTask.createHtml('fieldset', nodeParent);
    this.node = node;
    AudioFeedbackTask.createHtml('legend', node, null, this.data.title ? this.data.title : ('Frage ' + intNumber));
    if (this.data.html)
        AudioFeedbackTask.createHtml('div', node).innerHTML = this.data.html;
    var choices = this.data.choices;
    if (!this.data.keepOrder)
        choices = choices.map(function(v) { return { v: v, rand: Math.random() }; }).sort(function(a, b) { return a.rand - b.rand; }).map(function(o) { return o.v; });
    if (choices.length) {
        var ul = AudioFeedbackTask.createHtml('ol', node);
        var label, input, li;
        for (var z = 0; z < choices.length; ++z) {
            li = AudioFeedbackTask.createHtml('li', ul);
            label = AudioFeedbackTask.createHtml('label', li);
            input = AudioFeedbackTask.createHtml('input', AudioFeedbackTask.createHtml('span', label), { "name": "antwort" + intNumber, type: this.data.singlechoice ? 'radio' : 'checkbox' });
            AudioFeedbackTask.createHtml('div', label).innerHTML = choices[z].html;
            this._choices.push({ input: input, correct: choices[z].correct, response: choices[z].response });
        }
    }
}

/**
* Ggf. einen Korrekturhinweis ergänzen
* @return undefined
*/
FragenUI.prototype.addHint = function(
    hintHTML,       // Hinweistext (HTML)
    boolFirst       // Optional: Hinweis als erstes Element einfügen
) {
    if (!hintHTML)
        return;
    AudioFeedbackTask.createHtml('div', this.node, { 'class': 'hints' }).innerHTML = hintHTML;
}

/**
* Ein Eingabefeld durch die Lösung und Korrekturzeichen ersetzen
* @return Boolean   War die Antwort richtig?
*/
FragenUI._resolveInput = function(
    input,      // Eingabefeld
    correct     // Muss das Eingabefeld ausgewählt sein? (undefined für egal, true oder false)
) {
    var user = input.checked;
    var src = 'nix.svg';
    var alt = '✔✘☐☒';
    var result = false;
    switch ((user ? 4 : 0) + ((correct === true) ? 1 : ((correct === false) ? 0 : 2))) {
        case 1:
            src = 'dochx.svg';
            alt = '✘☐';
            break;
        case 4:
            src = 'dochnix.svg';
            alt = '✘☒';
            break;
        case 5:
        case 6:
            src = 'x.svg';
            alt = '✔☒';
            result = true;
            break;
        case 0:
        case 2:
        default:
            src = 'nix.svg';
            alt = '✔☐';
            result = true;
            break;
    }
    AudioFeedbackTask.createHtml('img', null, { alt: alt, src: 'share/' + src }, undefined, input);
    input.parentNode.parentNode.className = result ? 'correct' : 'wrong';
    input.parentNode.removeChild(input);
    return result;
}

/**
* Eine Frage korrigieren, die nur die Auswahl einer Auswahlmöglichkeit erlaubt
* @return Object    { correct: "Frage fehlerfrei beantwortet?", count: 1, score: "Richtig ausgewählte Antwortmöglichkeiten" }
*/
FragenUI.prototype._evaluateSinglechoice = function() {
    var correct = false;
    this._choices.map(function(choice) {
        if (choice.input.checked && choice.correct)
            correct = true;
    });
    for (var z = 0; z < this._choices.length; ++z) {
        if (this._choices[z].checked) {
            FragenUI._resolveInput(this._choices[z].input, this._choices[z].correct ? true : false) || this.addHint(this._choices[z].response);
        } else {
            FragenUI._resolveInput(this._choices[z].input, correct ? undefined : (this._choices[z].correct ? true : false)) || this.addHint(this._choices[z].response);
        }
    }
    return { "correct": correct, "count": 1, "score": (correct ? 1 : 0) };
}

/**
* Eine Frage korrigieren, die mehrfache Auswahl von Antwortmöglichkeiten erlaubt
* @return Object    { correct: "Frage fehlerfrei beantwortet?", count: "Anzahl an Antwortmöglichkeiten", score: "Richtig ausgewählte Antwortmöglichkeiten" }
*/
FragenUI.prototype._evaluateMultiplechoice = function() {
    var correct = true;
    var score = 0;
    for (var z = 0; z < this._choices.length; ++z) {
        if (FragenUI._resolveInput(this._choices[z].input, this._choices[z].correct ? true : false))
            ++score;
        else {
            correct = false;
            this.addHint(this._choices[z].response);
        }
    }
    return { "correct": correct, "count": this._choices.length, "score": score };
}

/**
* Frage korrigieren
* @return Object    { correct: "Frage fehlerfrei beantwortet?", count: "Erreichbare Punkteanzahl", score: "Erreichte Punkteanzahl" }
*/
FragenUI.prototype.evaluate = function() {
    var r = (this.data.singlechoice ? this._evaluateSinglechoice() : this._evaluateMultiplechoice());
    this.node.className = (r.correct ? 'correct' : 'wrong');
    if (!r.correct)
        this.addHint(this.data.response);
    return r;
}
