/******************************************************************************
* To the extent possible under law, Sebastian Roith
* <https://www.roith-unterrichtet.de> has waived all copyright and related or
* neighboring rights to this script. This work is published from Germany.
******************************************************************************/

AudioFeedbackTask.caching.push('nix.svg');
AudioFeedbackTask.caching.push('x.svg');
AudioFeedbackTask.caching.push('dochnix.svg');
AudioFeedbackTask.caching.push('x.svg');

/**
* Aus Fragen mit mehreren vorgegebenen Antwortmöglichkeiten bestehenden Test stellen
*/
function MultipleChoiceTest(
    data,       // Daten und Einstellungen für diesen Test
    parentNode  // HTMLElement, in dem der Test dargestellt werden soll
) {
    MultipleChoiceTest.parent.call(this);
    this.data = data;
    this.msgPlaceGeneral = new ScoTools.MessagePlace(true);
    this.msgPlaceGeneral.insertThisBefore(parentNode);
    this.msgPlace = new ScoTools.MessagePlace();
    this.msgPlace.insertThisBefore(parentNode);
    this.node = parentNode;
    this.establish();
    this.connect();
    AudioFeedbackTask.cache();
    this.renderTest();
}
MultipleChoiceTest.prototype = Object.create(AudioFeedbackTask.prototype);
MultipleChoiceTest.prototype.constructor = MultipleChoiceTest;
MultipleChoiceTest.parent = AudioFeedbackTask;

// Daten und Einstellungen für diesen Test
MultipleChoiceTest.prototype.data = {};

/**
* Fragendaten auswählen
* @return Array     Feld aus Objekten mit Infos zu tatsächlich zu stellenden Fragen
*/
MultipleChoiceTest.prototype.drawQuestions = function() {
    var a = this.data.questions.slice();
    // Fragengeneratoren anwenden
    if (this.data.generators instanceof Array)
        this.data.generators.map(function(generator) {
            if (typeof generator == 'function')
                generator.call(generator, a);
        });
    // Fragen auswählen
    a = a.map(function(value, index) {
        return { question: value, index: index, pos: Math.random() };
    }).sort(function(a, b) {
        return a.pos - b.pos;
    }).slice(0, this.data.questionsPerTest);
    if (this.data.keepOrder)
        a = a.sort(function(a, b) { return a.index - b.index; });
    return a.map(function(o) { return o.question; });
}

// Wörter für ganze Zahlen
MultipleChoiceTest.NUMBERS_AS_WORDS = ['null', 'eins', 'zwei', 'drei', 'vier', 'fünf', 'sechs', 'sieben', 'acht', 'neun', 'zehn', 'elf', 'zwölf'];

/**
* Aus Zahl und Einheit einen Satzteil zusammenbauen
* @return String    Satzteil
*/
MultipleChoiceTest.quantityToWords = function(
    count,      // Anzahl als natürliche Zahl
    plural,     // Mehrzahl der Einheit
    strOne      // Rückgabewert, wenn count == 1
) {
    if (count == 1)
        return strOne;
    return (MultipleChoiceTest.NUMBERS_AS_WORDS[count] ? MultipleChoiceTest.NUMBERS_AS_WORDS[count] : count) + ' ' + plural;
}

/**
* Test anzeigen
* @return undefined
*/
MultipleChoiceTest.prototype.renderTest = function() {
    var self = this;
    this.nodeMsg = null;
    this.node.innerHTML = '';
    if (this.data.htmlIntro)
        AudioFeedbackTask.createHtml('div', this.node).innerHTML = this.data.htmlIntro;
    this._form = AudioFeedbackTask.createHtml('form', this.node, { method: 'post', action: location.href });
    this._form.addEventListener('submit', function(ev) {
        ev.preventDefault();
        ev.stopPropagation();
        if (self.needRestart)
            self.renderTest();
        else
            self.evaluate();
    });
    this._questions = this.drawQuestions().map(function(q, index) { return new FragenUI(q, self._form, index + 1); });
    this._btn = AudioFeedbackTask.createHtml('input', AudioFeedbackTask.createHtml('div', this._form), { type: 'submit', value: 'Antworten überprüfen' });
    this.nodeBeforeMsg = this.node.firstChild;
    // Ggf. über Anzahl verbleibender Versuche informieren
    if (Number.isFinite(this.data.maxTries)) {
        var tries = this.getPreviousTries();
        if (tries >= this.data.maxTries)
            this.showMsg('Da du die maximal erlaubte Anzahl an Versuchen bereits erreicht hast, werden weitere Ergebnisse nicht mehr gespeichert', 'msg_warn');
        else if (this.data.reportScore || !this.getDone()) {
            switch (this.data.maxTries - tries) {
                case 1:
                    if (this.data.maxTries == 1)
                        this.showMsg('Dies ist dein einziger erlaubter Versuch', 'msg_warn');
                    else
                        this.showMsg('Dies ist dein letzter von ' + MultipleChoiceTest.quantityToWords(this.data.maxTries, 'erlaubten Versuchen'), 'msg_warn');
                    break;
                case 2:
                    this.showMsg('Dies ist dein vorletzter von ' + MultipleChoiceTest.quantityToWords(this.data.maxTries, 'erlaubten Versuchen'), 'msg_warn');
                    break;
                default:
                    this.showMsg('Du hast noch ' + MultipleChoiceTest.quantityToWords(this.data.maxTries - tries, '', 'einen') + ' von ' + MultipleChoiceTest.quantityToWords(this.data.maxTries, 'erlaubten Versuchen'), 'msg_info');
                    break;
            }
        }
    }
}

// Führt das Drücken der Schaltfläche zum Neustart (true) oder zum Überprüfen (false)
Object.defineProperty(MultipleChoiceTest.prototype, 'needRestart', {
    enumerable: true,
    set(v) {
        if (!this._btn)
            return;
        if (v)
            this._btn.value = 'Erneut versuchen';
        else
            this._btn.value = 'Antworten überprüfen';
        try {
            this._btn.animate([ { "transform": "scale(1)" }, { "transform": "scale(0)" }, { "transform": "scale(1)" } ], { "duration": 400 });
        } catch {}
    },
    get() {
        return (!this._btn || (this._btn.value == 'Erneut versuchen'));
    }
});

/**
* Aus Einzelergebnissen die Anteil der komplett richtigen Fragen ermitteln
* @return Number        Anteil der erreichten Punkte entsprechend countMode
*/
MultipleChoiceTest.calcRatioQuestions = function(
    arrResults      // Feld aus Rückgabewerten von FragenUI.evaluate()
) {
    return (arrResults.filter(function(r) { return r.correct; }).length / arrResults.length);
}

/**
* Aus Einzelergebnissen den durchschnittlichen Anteil an pro Frage richtig gesetzten Kreuzen berechnen
* @return Number        Anteil der erreichten Punkte entsprechend countMode
*/
MultipleChoiceTest.calcRatioAverage = function(
    arrResults      // Feld aus Rückgabewerten von FragenUI.evaluate()
) {
    return (arrResults.map(function(r) { return r.score / r.count; }).reduce(function(a, b) { return a + b; }, 0) / arrResults.length);
}

/**
* Aus Einzelergebnissen den Gesamtanteil an richtig gesetzten bzw. richtig nicht gesetzten Kreuzen pro setzbare Kreuze berechnen
* @return Number        Anteil der erreichten Punkte entsprechend countMode
*/
MultipleChoiceTest.calcRatioChoices = function(
    arrResults      // Feld aus Rückgabewerten von FragenUI.evaluate()
) {
    return (arrResults.map(function(r) { return r.score; }).reduce(function(a, b) { return a + b; }, 0) / arrResults.map(function(r) { return r.count; }).reduce(function(a, b) { return a + b; }, 0));
}

/**
* Aus Einzelergebnissen bei den Fragen entsprechend von countMode ein Gesamtergebnis berechnen
* @return Number        Anteil der erreichten Punkte entsprechend countMode
*/
MultipleChoiceTest.calcRatio = function(
    arrResults,     // Feld aus Rückgabewerten von FragenUI.evaluate()
    countMode       // Wie soll gezählt werden: "questions" (Standardwert), "average" oder "choices"
) {
    switch (countMode) {
        case 'average':
            return MultipleChoiceTest.calcRatioAverage(arrResults);
        case 'choices':
            return MultipleChoiceTest.calcRatioChoices(arrResults);
        case 'questions':
        default:
            return MultipleChoiceTest.calcRatioQuestions(arrResults);
    }
}

/**
* Anzahl der bisherigen Versuche auslesen
* @return Number    Anzahl der bisherigen Versuche
*/
MultipleChoiceTest.prototype.getPreviousTries = function() {
    var value =  this.read('suspendData');
    if ((value === undefined) || !(('' + value).length))
        value = this.read('score');
    value = +value;
    return (Number.isFinite(value) ? value : 0);
}

/**
* Ergebnis ggf. abspeichern und anzeigen, wenn der Anteil der richtigen Antworten zählt
* @return undefined
*/
MultipleChoiceTest.prototype._reportResultScore = function(
    bestanden,      // Bestanden oder nicht?
    ratio           // Anteil an erreichten Punkten
) {
    var score = Math.round(ratio * 100);
    var oldScore = undefined;
    var tries = this.getPreviousTries() + 1;
    var maxTries = Number.isFinite(this.data.maxTries) ? this.data.maxTries : Infinity;
    if (this.isConnected() && (this.read('lessonMode') == 'normal') && (tries <= maxTries)) {
        oldScore = this.read('score');
        this.saveScore(Math.max(score, oldScore), 0, 100, bestanden ? 'passed' : ((tries < maxTries) ? 'incomplete' : 'failed'));
    } else
        this.reportTime();
    if (bestanden || (tries >= maxTries))
        this.showFinalMessage();
    if (Number.isFinite(oldScore) && (oldScore < score))
        score += ' %, neuer Bestwert';
    else
        score += ' %';
    if (bestanden)
        this.showMsg('Test bestanden (' + score + ')', 'msg_success');
    else
        this.showMsg('Test nicht bestanden (' + score + ')', 'msg_failed');
}

/**
* Ergebnis ggf. abspeichern und anzeigen, wenn nur die Zahl der nötigen Versuche zählt
* @return undefined
*/
MultipleChoiceTest.prototype._reportResultTries = function(
    bestanden,      // Bestanden oder nicht?
    ratio           // Anteil an erreichten Punkten
) {
    var score = Math.round(ratio * 100);
    var tries = this.getPreviousTries() + 1;
    var maxTries = Number.isFinite(this.data.maxTries) ? this.data.maxTries : Infinity;
    if (this.canSave()) {
        if (bestanden)
            this.saveScore(Math.min(100, Math.max(51, 101 - tries)), 0, 100, 'passed');
        else
            this.saveScore(Math.max(1, Math.min(49, tries)), 0, 100, (tries < maxTries) ? 'incomplete' : 'failed');
        this.write('suspendData', tries);
    } else if (this.isConnected())
        this.reportTime();
    if (bestanden || (tries >= maxTries))
        this.showFinalMessage();
    if (bestanden)
        this.showMsg('Test bestanden (' + score + ' %)', 'msg_success');
    else
        this.showMsg('Test nicht bestanden (' + score + ' %)', 'msg_failed');
}

/**
* Richtigkeit der Antworten überprüfen
* @return undefined
*/
MultipleChoiceTest.prototype.evaluate = function() {
    var results = this._questions.map(function(question) { return question.evaluate(); });
    this.needRestart = true;
    var ratio = MultipleChoiceTest.calcRatio(results, this.data.countMode);
    var bestanden = (ratio >= ((this.data.threshold === undefined) ? 1 : this.data.threshold));
    if (this.reportScore)
        this._reportResultScore(bestanden, ratio);
    else
        this._reportResultTries(bestanden, ratio);
    this.playAudio(bestanden);
}
